/* Xscan: A HP scanner frontend for Unix written using Xforms.
 * Kees Lemmens; Nov. 1997
 */

#include <stdio.h>
#include <stdlib.h>
#include "forms.h"

#include "xscandefs.h"
#include "version.h"

static Xscanrc xscanInfo;

int CheckForExistence(char *filename)
{
   FILE *fp;
   
   if((fp=fopen(filename,"r")) != NULL)
   {
      fclose(fp);
      return 0;
   }
   else
     return 1;
}

char *UserResourceFile(int check)
{
   static char string[MAXSTRING];
   char *home = getenv("HOME");
   
   if(home != NULL)
     sprintf(string,"%s/.%s",home,XSCANRESOURCES);
   if(check)
   {
      if(CheckForExistence(string) != 0)
      {
	 fl_show_message("No resources file found.",
			 "Using precompiled defaults.","");
	 *string = 0;
      }
   }
   return string;
}

char *ReadString(const char *label,char *value,FILE *rcfile)
{  char string[MAXSTRING];
   char *ptr;
   int count;
   rewind(rcfile);
   
   while(1)
   {
      fgets(string,MAXSTRING,rcfile);
      if(feof(rcfile))
      {
	 *value = 0;
	 return NULL;
      }
      
      /* Include ":" in compare string by setting count 1 higher to avoid
       * confusing resources starting with the same string ! */
      for(ptr=string,count=1; *ptr != ':' && *ptr != '\0';ptr++,count++);
      if(! strncmp(string,label,count))
      {
	 sscanf(string,label,value);
	 break;
      }
   }
   
   return value;
}

void WriteString(const char *label,const char *value,FILE *rcfile)
{
   if(strlen(value) > 0)
     fprintf(rcfile,label,value);
}

Xscanrc *LoadUserInfo()
{
   FILE *rcfile;
   Xscanrc *info = &xscanInfo;

   if((rcfile = fopen(UserResourceFile(1),"r")) != NULL)
   {   
      ReadString("ScanPrint: %[^\n]",info->ScanPrint,rcfile);
      ReadString("SpoolDir:  %[^\n]",info->SpoolDir,rcfile);
      ReadString("ScanView:  %[^\n]",info->ScanView,rcfile);
      ReadString("PpmToGif:  %[^\n]",info->PpmToGif,rcfile);
      ReadString("PpmToPs:   %[^\n]",info->PpmToPs,rcfile);
      ReadString("Formats:   %[^\n]",info->Formats,rcfile);
      ReadString("ScanCmd:   %[^\n]",info->ScanCmd,rcfile);
      ReadString("PreviewCmd: %[^\n]",info->PreviewCmd,rcfile);
      ReadString("FinalCmd:   %[^\n]",info->FinalCmd,rcfile);
      ReadString("BrightnessOffset: %[^\n]",info->BrightnessOffset,rcfile);
      ReadString("BrightnessFactor: %[^\n]",info->BrightnessFactor,rcfile);
      ReadString("ContrastOffset:   %[^\n]",info->ContrastOffset,rcfile);
      ReadString("ContrastFactor:   %[^\n]",info->ContrastFactor,rcfile);
      fclose(rcfile);
   }
   else
   {   
      strcpy(info->ScanPrint , SCANPRINT);
      strcpy(info->SpoolDir  , SPOOLDIR);
      strcpy(info->ScanView  , SCANVIEW );
      strcpy(info->PpmToGif  , PPMTOGIF);
      strcpy(info->PpmToPs   , PPMTOPS);
      strcpy(info->Formats   , FORMATS);
      strcpy(info->ScanCmd   , SCANCMD);
      strcpy(info->PreviewCmd , PREVIEWCMD);
      strcpy(info->FinalCmd   , FINALCMD );
      strcpy(info->BrightnessOffset , BRGTNS_OFFSET);
      strcpy(info->BrightnessFactor , BRGTNS_FACTOR);
      strcpy(info->ContrastOffset   , CNTRST_OFFSET);
      strcpy(info->ContrastFactor   , CNTRST_FACTOR);
   }
   return info;
}

void SaveUserInfo(const Xscanrc *info)
{
   FILE *rcfile;
   char *string;
   
   string = UserResourceFile(0);
   if(strlen(string) == 0 || (rcfile = fopen(string,"w")) == NULL)
   {  fl_show_message("","Alert: Can't write resource information !","");
      return;
   }

   WriteString("ScanPrint: %s\n",info->ScanPrint,rcfile);
   WriteString("SpoolDir:  %s\n",info->SpoolDir,rcfile);
   WriteString("ScanView:  %s\n",info->ScanView,rcfile);
   WriteString("PpmToGif:  %s\n",info->PpmToGif,rcfile);
   WriteString("PpmToPs:   %s\n",info->PpmToPs,rcfile);
   WriteString("Formats:   %s\n",info->Formats,rcfile);
   WriteString("ScanCmd:   %s\n",info->ScanCmd,rcfile);
   WriteString("PreviewCmd: %s\n",info->PreviewCmd,rcfile);
   WriteString("FinalCmd:   %s\n",info->FinalCmd,rcfile);
   WriteString("BrightnessOffset: %s\n",info->BrightnessOffset,rcfile);
   WriteString("BrightnessFactor: %s\n",info->BrightnessFactor,rcfile);
   WriteString("ContrastOffset:   %s\n",info->ContrastOffset,  rcfile);
   WriteString("ContrastFactor:   %s\n",info->ContrastFactor,  rcfile);
   
   fclose(rcfile);
}

 void InitSystemWin(FD_SystemWin *fw)
{
   fw->vdata = (void *)info;
   
   fl_deactivate_form(fd_Xscanner->Xscanner);
   
   fl_set_input(fw->ScanPrint,info->ScanPrint);
   fl_set_input(fw->SpoolDir, info->SpoolDir);
   fl_set_input(fw->ScanView, info->ScanView);
   fl_set_input(fw->PpmToGif, info->PpmToGif);
   fl_set_input(fw->PpmToPs,  info->PpmToPs);
   fl_set_input(fw->Formats,  info->Formats);
   fl_set_input(fw->ScanCmd,  info->ScanCmd);
   fl_set_input(fw->PreviewCmd,info->PreviewCmd);
   fl_set_input(fw->FinalCmd,  info->FinalCmd);
   fl_set_input(fw->BrightnessOffset,info->BrightnessOffset);
   fl_set_input(fw->BrightnessFactor,info->BrightnessFactor);
   fl_set_input(fw->ContrastOffset  ,info->ContrastOffset);
   fl_set_input(fw->ContrastFactor  ,info->ContrastFactor);

   fl_show_form(fw->SystemWin,FL_PLACE_CENTER,FL_TRANSIENT,"SystemWin");
}

void SystemOK(FL_OBJECT *ob, long data)
{
   Xscanrc *info = (Xscanrc *)fd_SystemWin->vdata;
   
   strncpy(info->ScanPrint,fl_get_input(fd_SystemWin->ScanPrint),MAXSTRING-1);
   strncpy(info->SpoolDir ,fl_get_input(fd_SystemWin->SpoolDir) ,MAXSTRING-1);
   strncpy(info->ScanView ,fl_get_input(fd_SystemWin->ScanView) ,MAXSTRING-1);
   strncpy(info->PpmToGif ,fl_get_input(fd_SystemWin->PpmToGif) ,MAXSTRING-1);
   strncpy(info->PpmToPs  ,fl_get_input(fd_SystemWin->PpmToPs)  ,MAXSTRING-1);
   strncpy(info->Formats  ,fl_get_input(fd_SystemWin->Formats)  ,MAXSTRING-1);
   strncpy(info->ScanCmd  ,fl_get_input(fd_SystemWin->ScanCmd)  ,MAXSTRING-1);
   strncpy(info->PreviewCmd,fl_get_input(fd_SystemWin->PreviewCmd),MAXSTRING-1);
   strncpy(info->FinalCmd,  fl_get_input(fd_SystemWin->FinalCmd),  MAXSTRING-1);
   strncpy(info->BrightnessOffset,fl_get_input(fd_SystemWin->BrightnessOffset),MAXSTRING-1);
   strncpy(info->BrightnessFactor,fl_get_input(fd_SystemWin->BrightnessFactor),MAXSTRING-1);
   strncpy(info->ContrastOffset  ,fl_get_input(fd_SystemWin->ContrastOffset),  MAXSTRING-1);
   strncpy(info->ContrastFactor  ,fl_get_input(fd_SystemWin->ContrastFactor),  MAXSTRING-1);

   SaveUserInfo(info);
   
   fl_hide_form(fd_SystemWin->SystemWin);
   fl_activate_form(fd_Xscanner->Xscanner);
}

void SystemCancel(FL_OBJECT *ob, long data)
{
   fl_hide_form(fd_SystemWin->SystemWin);
   fl_activate_form(fd_Xscanner->Xscanner);
}
