/* Xscanner: A generic scanner frontend for Unix written using Xforms.
 * Kees Lemmens; Nov. 1997
 * Modified Jan 2001 to support SANE as well
 */

// #define DEBUG 1

/* Add your own backend specifications if necessary !! */
// #define SANE_HPSCAN  1
#define SANE_EPSON   1
//  #define HPSCANPBM 1

#include "xscanner.h"

#define XSCANRESOURCES  "xscanrc"

#define SCANPRINT "pnmtops -rle %s | lpps"
#define SCANVIEW  "xv %s"
#define PPMTOGIF  "ppmtogif %s > %s"
#define PPMTOPS   "pnmtops -rle %s > %s"
#define SPOOLDIR  "/tmp"

/* SANE HP Scanjet backend options */
#ifdef SANE_HPSCAN

# define SCANCMD "scanimage" /* SANE cmdline tool for LINUX */
# define HASBRIGHTNESS
# define BRGTNS_OFFSET "0"
# define BRGTNS_FACTOR "127"

# define HASCONTRAST 
# define CNTRST_OFFSET "0"
# define CNTRST_FACTOR "127"

# define FORMATS " Grayscale | Color | Halftone | Lineart "
# define PREVIEWCMD "%s --mode Grayscale --resolution 75 -y %f "       \
   "--preview=yes --format=pnm --contrast %.0f --brightness %.0f | "   \
     "pnmscale 0.46 | ppmtoxpm > %s"
# define FINALCMD  "%s --mode %s --resolution %d -x %f -y %f "         \
   "--contrast %.0f --brightness %.0f > %s"
#endif

/* SANE Epson Perfection 610 scanner backend options */
#ifdef SANE_EPSON

# define SCANCMD "scanimage" /* SANE cmdline tool for LINUX */
# define HASBRIGHTNESS   // External !
# define BRGTNS_OFFSET "1.00001"
# define BRGTNS_FACTOR "1"

# define CNTRST_OFFSET "N/A"
# define CNTRST_FACTOR "N/A"

/*
 Note that I use an external program for brightness control 
 as long as the backend doesn't support it :
 */

# define FORMATS " Color | Gray | Binary "
# define PREVIEWCMD "%s --mode Gray --resolution 75 -y %f "      \
   "--preview=yes --format=pnm | pnmgamma %f |"                  \
     "pnmscale 0.46 | ppmtoxpm > %s"
# define FINALCMD   "%s --mode %s --resolution %d -x %f -y %f |" \
   " pnmgamma %f > %s"
#endif

/* Add your own SANE backend defs here if you like : */

/* Support for good old HPSCANPBM backend */
#ifdef HPSCANPBM

# define SCANCMD "hpscanpbm" /* HP Scanjet cmdline tool for HPUX and LINUX */
# define HASBRIGHTNESS
# define BRGTNS_OFFSET "0"
# define BRGTNS_FACTOR "1"

# define HASCONTRAST 
# define CNTRST_OFFSET "0"
# define CNTRST_FACTOR "1"

# define FORMATS " greyscale | color | thresholded | dithered "
# define PREVIEWCMD "%s -format greyscale -dpi 75 -mm "       \
   "-xres 300 -yres 400 -height %f -cont %f -bright %f | "    \
     "ppmtoxpm > %s"
# define FINALCMD   "%s -format %s -dpi %d -mm "              \
   "-width %f -height %f -cont %f -bright %f -o %s"
#endif

/* Relation between preview area and real scan area :
 It's slightly more than A4 dims to ensure we really 
 catch everything we selected with the crosshair.
 */
#define XFACTOR  216
#define YFACTOR  298

#define MAXCMD      4000
#define MAXLINE      256
#define MAXSTRING    256

typedef struct 
{
   char SpoolDir [MAXSTRING];
   char ScanPrint[MAXSTRING];
   char ScanView [MAXSTRING];
   char PpmToGif [MAXSTRING];
   char PpmToPs  [MAXSTRING];
   char Formats  [MAXSTRING];
   char ScanCmd  [MAXSTRING];
   char PreviewCmd[MAXSTRING];
   char FinalCmd  [MAXSTRING];
   char BrightnessOffset [MAXSTRING];
   char BrightnessFactor [MAXSTRING];
   char ContrastOffset   [MAXSTRING];
   char ContrastFactor   [MAXSTRING];

   char *Scanname;
   int  ExitFlag;
} Xscanrc;

extern FD_Xscanner *fd_Xscanner;
extern FD_LogView *fd_LogView;
extern FD_SystemWin *fd_SystemWin;
extern Xscanrc *info;

/* External function declarations */

char    *BuildResourceFile();
char    *ReadString(const char *label,char *string,FILE *rcfile);
Xscanrc *LoadUserInfo();
void    SaveUserInfo(const Xscanrc *info);
void    InitSystemWin(FD_SystemWin *fd_SystemWin);
void    RedirectIO(FILE *iohandle);
void    SpawnCmd(char *cmdbuffer,int flag);
void    RemoveTmpdir(char *dirname);
void    LogViewOpen(FL_OBJECT *ob, long data);
