/* Xscanner: A scanner frontend for Unix written using Xforms.
 * Kees Lemmens; Nov. 1997
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>

#include "forms.h"

#include "xscandefs.h"
#include "version.h"

void SpawnCmd(char *cmdbuffer,int flag)
{  pid_t pid;
   
   if ((pid=fork()) == 0)
   {
      fprintf(stderr,"Executing: %s\n",cmdbuffer);
      setsid();
      system(cmdbuffer);
      exit(0);
   }
   else
   {
      if(flag)
	wait(NULL);
      else
	signal(SIGCHLD,SIG_IGN);
   }
}

int CheckForScan(char *Scanname)
{
   if(Scanname == NULL)
   {
      fl_show_message("","No scan is available yet:",
		      "You should make a scan first !");
      return 1;
   }
   return 0;
}

void OnelinerMsg(char *string)
{
   int km;
   FL_Coord x,y;

   fl_get_mouse(&x,&y,&km);
   fl_show_oneliner(string,x,y);
   fl_check_forms();
}

void FileMenu(FL_OBJECT *ob, long data)
{
   char cmdbuffer[MAXCMD];
   char *ExportCmd;
   char *Output;
   
   switch(fl_get_menu(ob))
   {
    case 1: case 2: case 3:
      if(CheckForScan(info->Scanname))
	return;
      switch(fl_get_menu(ob))
      {
       case 1: 
	 Output = (char *)fl_show_fselector("Set Outputfile","",
					    "*.gif","xscanner.gif");
	 ExportCmd  = info->PpmToGif;
	 break;
       case 2:
	 Output = (char *)fl_show_fselector("Set Outputfile","",
					    "*.ppm","xscanner.ppm");
	 ExportCmd  = "cp %s %s";
	 break;
       default : /* case 3 */
	 Output = (char *)fl_show_fselector("Set Outputfile","",
					    "*.ps","xscanner.ps");
	 ExportCmd  = info->PpmToPs;
	 break;
      }

      sprintf(cmdbuffer,ExportCmd,info->Scanname,Output);
      OnelinerMsg("Now exporting scanned image ...");
      SpawnCmd(cmdbuffer,1); /* fork and wait !! */
      fl_hide_oneliner();
      info->ExitFlag = 0;
      break;
      
    case 4:
      fl_call_object_callback(fd_Xscanner->Exit);
      break;
    default: /* Can't happen !! */ break;
   }
}

void OptionsMenu(FL_OBJECT *ob, long data)
{
   switch(fl_get_menu(ob))
   {
    case 1:  InitSystemWin(fd_SystemWin); break;
    case 2:  LogViewOpen((FL_OBJECT *)NULL,(long)0); break;
    default: /* Can't happen !! */ break;
   }
}

void RedrawPreviewArea(FL_OBJECT *ob, long data)
{
   double sx,sy;
   FL_COORD ax,ay,aw,ah;
   int xinset = 4,yinset = 2; // stay away from the edges 
   
   if(fl_get_button(fd_Xscanner->UseCrosshair))
   {
      fl_show_object(fd_Xscanner->Crosshair);
      sx =       fl_get_positioner_xvalue(fd_Xscanner->Crosshair);
      sy = 1.0 - fl_get_positioner_yvalue(fd_Xscanner->Crosshair);

      fl_get_object_geometry(fd_Xscanner->PreviewArea,&ax,&ay,&aw,&ah);
      fl_set_object_size(fd_Xscanner->SubAreaBox,aw*sx-xinset,ah*sy-yinset);
      
      fl_redraw_object(fd_Xscanner->PreviewAreaBox); // Box
      fl_redraw_object(fd_Xscanner->PreviewArea);    // Pixmap
      fl_show_object(fd_Xscanner->SubAreaBox);       // blue frame
   }
   else
   {
      fl_hide_object(fd_Xscanner->SubAreaBox);       // blue frame
      fl_hide_object(fd_Xscanner->Crosshair);
      fl_redraw_object(fd_Xscanner->PreviewAreaBox); // Box
      fl_redraw_object(fd_Xscanner->PreviewArea);    // Pixmap
   }
}

void Preview(FL_OBJECT *ob, long data)
{
   double br_y;
   char *SpoolDir;
   char *Tmpnam;
   char cmdbuffer[MAXCMD];
   
   SpoolDir = (strlen(info->SpoolDir) == 0) ? SPOOLDIR:info->SpoolDir;
   Tmpnam = tempnam(SpoolDir,"pview");

   if(fl_get_button(fd_Xscanner->FullSize)==1)
      br_y=1.0;
   else
      br_y = 1.0 - fl_get_positioner_yvalue(fd_Xscanner->Crosshair);
   
   br_y *= YFACTOR;

   sprintf(cmdbuffer,info->PreviewCmd,info->ScanCmd,
	   br_y,
#ifdef HASCONTRAST	  
	   (fl_get_slider_value(fd_Xscanner->Contrast)   +
	    atoi(info->ContrastOffset)) * atoi(info->ContrastFactor),
#endif
#ifdef HASBRIGHTNESS
	   (fl_get_slider_value(fd_Xscanner->Brightness) +
	    atoi(info->BrightnessOffset)) * atoi(info->BrightnessFactor),
#endif
	   Tmpnam);
   
   OnelinerMsg("Now scanning preview image ...");

   SpawnCmd(cmdbuffer,1);  /* fork, but wait until finished !! */
   
   fl_freeze_form(fd_Xscanner->Xscanner);
   fl_set_pixmap_align(fd_Xscanner->PreviewArea,
		       FL_ALIGN_TOP|FL_ALIGN_INSIDE,0,0);
   fl_set_pixmap_file(fd_Xscanner->PreviewArea,Tmpnam);
   fl_unfreeze_form(fd_Xscanner->Xscanner);
   fl_hide_oneliner();
   fl_call_object_callback(fd_Xscanner->FullSize); // just to force a redraw 
   
   info->ExitFlag = 1;
#ifndef DEBUG
   remove(Tmpnam);
#endif
}

void Final(FL_OBJECT *ob, long data)
{
   double br_x,br_y;
   char *SpoolDir;
   char cmdbuffer[MAXCMD];
   static char Scanname[MAXSTRING];
   
   if(info->Scanname != NULL)
     remove(info->Scanname); /* remove previous scan */

   SpoolDir = (strlen(info->SpoolDir) == 0) ? SPOOLDIR:info->SpoolDir;
   strncpy(Scanname,tempnam(SpoolDir,"scan"),MAXSTRING-1);
      
   info->Scanname = Scanname;
   
   if(fl_get_button(fd_Xscanner->FullSize)==1)
   {
      br_x=1.0;
      br_y=1.0;
   }
   else
   {
      br_x =       fl_get_positioner_xvalue(fd_Xscanner->Crosshair);
      br_y = 1.0 - fl_get_positioner_yvalue(fd_Xscanner->Crosshair);
   }
   br_x *= XFACTOR; br_y *= YFACTOR;

   sprintf(cmdbuffer,info->FinalCmd,info->ScanCmd,
	   fl_get_choice_text(fd_Xscanner->Format),
	   (int)fl_get_slider_value(fd_Xscanner->Resolution),
	   br_x,br_y,
#ifdef HASCONTRAST	  
	   (fl_get_slider_value(fd_Xscanner->Contrast)   +
	    atoi(info->ContrastOffset)) * atoi(info->ContrastFactor),
#endif
#ifdef HASBRIGHTNESS
	   (fl_get_slider_value(fd_Xscanner->Brightness) +
	    atoi(info->BrightnessOffset)) * atoi(info->BrightnessFactor),
#endif
	   info->Scanname);
   
   OnelinerMsg("Now scanning final image ... \n");
   
   SpawnCmd(cmdbuffer,1);  /* fork, but wait until finished !! */
   fl_hide_oneliner();
   info->ExitFlag = 1;
}

void View(FL_OBJECT *ob, long data)
{
   char cmdbuffer[MAXCMD];
   char *ViewCmd;
   
   ViewCmd = (strlen(info->ScanView) == 0) ? SCANVIEW:info->ScanView;
   
   if(CheckForScan(info->Scanname))
     return;
   
   sprintf(cmdbuffer,ViewCmd,info->Scanname);
   
   OnelinerMsg("Starting viewer using preset view command ");

   SpawnCmd(cmdbuffer,0); /* fork, no wait !! */
   sleep(3);
   fl_hide_oneliner();
}

void Print(FL_OBJECT *ob, long data)
{
   char cmdbuffer[MAXCMD];
   char *PrintCmd;
   
   PrintCmd = (strlen(info->ScanPrint) == 0) ? SCANPRINT:info->ScanPrint;
 
   if(CheckForScan(info->Scanname))
     return;
   
   sprintf(cmdbuffer,PrintCmd,info->Scanname);
   
   OnelinerMsg("Your scan was printed using the preset print command");
   
   SpawnCmd(cmdbuffer,0); /* fork, no wait !! */
   sleep(3);
   fl_hide_oneliner();
}

void Exit(FL_OBJECT *ob, long data)
{
   if(info->ExitFlag)
     if(! fl_show_question("Abandon current scan ?",1))
       return;
#ifndef DEBUG
   if(info->Scanname != NULL)
     remove(info->Scanname);
#endif
   exit(0);
}

void Info(FL_OBJECT *ob, long data)
{
   fl_show_message
     ("Xscanner: a SANE/hpscanpbm frontend using the Xforms library",
      "by Kees Lemmens, The Netherlands (C) 1997-2001",
      VERSION ", Faculty TWI, TU Delft");
}

